﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 클래스 계층 트리 뷰
    /// </summary>
    public class ClassHierarchyTreeView : TreeView
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ClassHierarchyTreeView(rootType)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="rootType">루트 타입</param>
        public ClassHierarchyTreeView(Type rootType)
        {
            UIElement dummyUIElement = new UIElement();

            List<Assembly> assemblyList = new List<Assembly>();

            AssemblyName[] assemblyNameArray = Assembly.GetExecutingAssembly().GetReferencedAssemblies();

            foreach(AssemblyName assemblyName in assemblyNameArray)
            {
                assemblyList.Add(Assembly.Load(assemblyName));
            }

            SortedList<string, Type> classSortedList = new SortedList<string, Type>();

            classSortedList.Add(rootType.Name, rootType);

            foreach(Assembly assembly in assemblyList)
            {
                foreach(Type type in assembly.GetTypes())
                {
                    if(type.IsPublic && type.IsSubclassOf(rootType))
                    {
                        classSortedList.Add(type.Name, type);
                    }
                }
            }

            TypeTreeViewItem rootTypeTreeViewItem = new TypeTreeViewItem(rootType);

            Items.Add(rootTypeTreeViewItem);

            CreateDerivedClasses(rootTypeTreeViewItem, classSortedList);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 파생 클래스 생성하기 - CreateDerivedClasses(baseTypeTreeViewItem, classSortedList)

        /// <summary>
        /// 파생 클래스 생성하기
        /// </summary>
        /// <param name="baseTypeTreeViewItem">베이스 타입 트리 뷰 항목</param>
        /// <param name="classSortedList">클래스 정렬 리스트</param>
        private void CreateDerivedClasses(TypeTreeViewItem baseTypeTreeViewItem, SortedList<string, Type> classSortedList)
        {
            foreach(KeyValuePair<string, Type> keyValuePair in classSortedList)
            {
                if(keyValuePair.Value.BaseType == baseTypeTreeViewItem.Type)
                {
                    TypeTreeViewItem typeTreeViewItem = new TypeTreeViewItem(keyValuePair.Value);

                    baseTypeTreeViewItem.Items.Add(typeTreeViewItem);

                    CreateDerivedClasses(typeTreeViewItem, classSortedList);
                }
            }
        }

        #endregion
    }
}